@@ -1,7 +1,5 @@
 """ACME utilities."""
-import six
-
 
 def map_keys(dikt, func):
     """Map dictionary keys."""
-    return {func(key): value for key, value in six.iteritems(dikt)}
+    return {func(key): value for key, value in dikt.items()}
